@@ -143,6 +143,14 @@ printf "<input type=radio name=remote value=1 %s> $text{'yes'}\n",
 printf "<input type=radio name=remote value=0 %s> $text{'no'}</td> </tr>\n",
 	$_[0]->{'remote'} ? "" : "checked";
 
+print "<tr> <td><b>$text{'acl_slaves'}</b></td> <td nowrap>\n";
+printf "<input type=radio name=slaves value=1 %s> $text{'yes'}\n",
+	$_[0]->{'slaves'} ? "checked" : "";
+printf "<input type=radio name=slaves value=0 %s> $text{'no'}</td>\n",
+	$_[0]->{'slaves'} ? "" : "checked";
+
+print "</tr>\n";
+
 print "<tr> <td><b>$text{'acl_views'}</b></td> <td colspan=3>\n";
 printf "<input type=radio name=views value=1 %s> $text{'yes'}\n",
 	$_[0]->{'views'} == 1 ? "checked" : "";
@@ -205,6 +213,7 @@ $_[0]->{'params'} = $in{'params'};
 $_[0]->{'opts'} = $in{'opts'};
 $_[0]->{'delete'} = $in{'delete'};
 $_[0]->{'findfree'} = $in{'findfree'};
+$_[0]->{'slaves'} = $in{'slaves'};
 $_[0]->{'views'} = $in{'views'};
 $_[0]->{'remote'} = $in{'remote'};
 $_[0]->{'gen'} = $in{'gen'};
