@@ -58,7 +58,7 @@ return undef;
 sub post_restore
 {
 local $pidfile = &get_pid_file();
-if (&check_pid_file(&make_chroot($pidfile))) {
+if (&check_pid_file(&make_chroot($pidfile, 1))) {
 	return &restart_bind();
 	}
 return undef;
