@@ -1,7 +1,5 @@
 # bind8-lib.pl
 # Common functions for bind8 config files
-# XXX get_config_parent should take file argument, for use by create_view.cgi
-# XXX other create_ scripts should not add direct to file!
 
 do '../web-lib.pl';
 &init_config();
@@ -10,6 +8,9 @@ require '../ui-lib.pl';
 @extra_forward = split(/\s+/, $config{'extra_forward'});
 @extra_reverse = split(/\s+/, $config{'extra_reverse'});
 %is_extra = map { $_, 1 } (@extra_forward, @extra_reverse);
+%access = &get_module_acl();
+$zone_names_cache = "$module_config_directory/zone-names";
+$zone_names_version = 2;
 
 if (open(VERSION, "$module_config_directory/version")) {
 	chop($bind_version = <VERSION>);
@@ -27,16 +28,25 @@ if (!@get_config_cache) {
 return \@get_config_cache;
 }
 
-# get_config_parent()
+# get_config_parent([file])
 # Returns a structure containing the top-level config as members
 sub get_config_parent
 {
-local $conf = &get_config();
-return { 'file' => $config{'named_conf'},
-	 'type' => 1,
-	 'line' => -1,
-	 'eline' => $lines_count{$config{'named_conf'}},
-	 'members' => $conf };
+local $file = $_[0] || $config{'named_conf'};
+if (!defined($get_config_parent_cache{$file})) {
+	local $conf = &get_config();
+	if (!defined($lines_count{$file})) {
+		local $lref = &read_file_lines($file);
+		$lines_count{$file} = @$lref;
+		}
+	$get_config_parent_cache{$file} =
+	       { 'file' => $file,
+		 'type' => 1,
+		 'line' => -1,
+		 'eline' => $lines_count{$file},
+		 'members' => $conf };
+	}
+return $get_config_parent_cache{$file};
 }
 
 # read_config_file(file, [expand includes])
@@ -217,18 +227,27 @@ elsif (wantarray) { return map { $_->{'value'} } @v; }
 else { return $v[0]->{'value'}; }
 }
 
-# base_directory([&config])
+# base_directory([&config], [no-cache])
 # Returns the base directory for named files
 sub base_directory
 {
-local($opts, $dir, $conf);
-$conf = $_[0] ? $_[0] : &get_config();
-if (($opts = &find("options", $conf)) &&
-    ($dir = &find("directory", $opts->{'members'}))) {
-	return $dir->{'value'};
+if ($_[1]) {
+	# Actually work out base
+	local ($opts, $dir, $conf);
+	$conf = $_[0] ? $_[0] : &get_config();
+	if (($opts = &find("options", $conf)) &&
+	    ($dir = &find("directory", $opts->{'members'}))) {
+		return $dir->{'value'};
+		}
+	$config{'named_conf'} =~ /^(.*)\/[^\/]+$/;
+	return $1;
+	}
+else {
+	# Use cache
+	local %znc;
+	&read_file_cached($zone_names_cache, \%znc);
+	return $znc{'base'} || &base_directory($_[0], 1);
 	}
-$config{'named_conf'} =~ /^(.*)\/[^\/]+$/;
-return $1;
 }
 
 # save_directive(&parent, name|&old, &values, indent, [structonly])
@@ -247,6 +266,8 @@ for($i=0; $i<@oldv || $i<@newv; $i++) {
 		# a new directive is being added.. put it at the end of
 		# the parent
 		if (!$_[4]) {
+			local $parent = &get_config_parent($newv[$i]->{'file'} ||
+							   $_[0]->{'file'});
 			$lref = &read_file_lines(
 				&make_chroot($newv[$i]->{'file'} ||
 					     $_[0]->{'file'}));
@@ -256,7 +277,7 @@ for($i=0; $i<@oldv || $i<@newv; $i++) {
 			$newv[$i]->{'line'} = $_[0]->{'eline'};
 			$newv[$i]->{'eline'} =
 				$_[0]->{'eline'} + scalar(@nl) - 1;
-			&renumber(&get_config(), $_[0]->{'eline'},
+			&renumber($parent, $_[0]->{'eline'}-1,
 				  $_[0]->{'file'}, scalar(@nl));
 			}
 		push(@$pm, $newv[$i]);
@@ -265,6 +286,8 @@ for($i=0; $i<@oldv || $i<@newv; $i++) {
 		# a new directive is being added.. put it at the start of
 		# the parent
 		if (!$_[4]) {
+			local $parent = &get_config_parent($newv[$i]->{'file'} ||
+							   $_[0]->{'file'});
 			$lref = &read_file_lines(
 				&make_chroot($newv[$i]->{'file'} ||
 					     $_[0]->{'file'}));
@@ -274,7 +297,7 @@ for($i=0; $i<@oldv || $i<@newv; $i++) {
 			$newv[$i]->{'line'} = $_[0]->{'line'}+1;
 			$newv[$i]->{'eline'} =
 				$_[0]->{'line'} + scalar(@nl);
-			&renumber(&get_config(), $_[0]->{'line'},
+			&renumber($parent, $_[0]->{'line'},
 				  $_[0]->{'file'}, scalar(@nl));
 			}
 		splice(@$pm, 0, 0, $newv[$i]);
@@ -282,10 +305,12 @@ for($i=0; $i<@oldv || $i<@newv; $i++) {
 	elsif ($i >= @newv) {
 		# a directive was deleted
 		if (!$_[4]) {
-			$lref = &read_file_lines(&make_chroot($oldv[$i]->{'file'}));
+			local $parent = &get_config_parent($oldv[$i]->{'file'});
+			$lref = &read_file_lines(
+					&make_chroot($oldv[$i]->{'file'}));
 			$ol = $oldv[$i]->{'eline'} - $oldv[$i]->{'line'} + 1;
 			splice(@$lref, $oldv[$i]->{'line'}, $ol);
-			&renumber(&get_config(), $oldv[$i]->{'eline'},
+			&renumber($parent, $oldv[$i]->{'eline'},
 				  $oldv[$i]->{'file'}, -$ol);
 			}
 		splice(@$pm, &indexof($oldv[$i], @$pm), 1);
@@ -293,7 +318,9 @@ for($i=0; $i<@oldv || $i<@newv; $i++) {
 	else {
 		# updating some directive
 		if (!$_[4]) {
-			$lref = &read_file_lines(&make_chroot($oldv[$i]->{'file'}));
+			local $parent = &get_config_parent($oldv[$i]->{'file'});
+			$lref = &read_file_lines(
+					&make_chroot($oldv[$i]->{'file'}));
 			@nl = &directive_lines($newv[$i], $_[3]);
 			$ol = $oldv[$i]->{'eline'} - $oldv[$i]->{'line'} + 1;
 			splice(@$lref, $oldv[$i]->{'line'}, $ol, @nl);
@@ -301,7 +328,7 @@ for($i=0; $i<@oldv || $i<@newv; $i++) {
 			$newv[$i]->{'line'} = $oldv[$i]->{'line'};
 			$newv[$i]->{'eline'} =
 				$oldv[$i]->{'line'} + scalar(@nl) - 1;
-			&renumber(&get_config(), $oldv[$i]->{'eline'},
+			&renumber($parent, $oldv[$i]->{'eline'},
 				  $oldv[$i]->{'file'}, scalar(@nl) - $ol);
 			}
 		$pm->[&indexof($oldv[$i], @$pm)] = $newv[$i];
@@ -333,19 +360,20 @@ $rv[$#rv] .= ";";
 return @rv;
 }
 
-# renumber(&directives, line, file, count)
+# renumber(&parent, line, file, count)
 # Runs through the given array of directives and increases the line numbers
 # of all those greater than some line by the given count
 sub renumber
 {
-local($d);
-foreach $d (@{$_[0]}) {
-	if ($d->{'file'} eq $_[2]) {
-		if ($d->{'line'} > $_[1]) { $d->{'line'} += $_[3]; }
-		if ($d->{'eline'} > $_[1]) { $d->{'eline'} += $_[3]; }
-		}
-	if ($d->{'type'}) {
-		&renumber($d->{'members'}, $_[1], $_[2], $_[3]);
+if ($_[0]->{'file'} eq $_[2]) {
+	if ($_[0]->{'line'} > $_[1]) { $_[0]->{'line'} += $_[3]; }
+	if ($_[0]->{'eline'} > $_[1]) { $_[0]->{'eline'} += $_[3]; }
+	}
+if ($_[0]->{'type'}) {
+	# Do sub-members
+	local $d;
+	foreach $d (@{$_[0]->{'members'}}) {
+		&renumber($d, $_[1], $_[2], $_[3]);
 		}
 	}
 }
@@ -572,11 +600,13 @@ local($conf, @zl, $rev, $z, $revconf, $revfile, $revrec, @revrecs, $addr, $rr,
       @octs, $i, @hexs, $ipv6, @zero);
 
 # find reverse domain
-$conf = &get_config();
-if ($in{'view'} ne '') {
-	$conf = $conf->[$in{'view'}]->{'members'};
+local @zl = grep { $_->{'type'} ne 'view' } &list_zone_names();
+if ($_[1] ne '') {
+	@zl = grep { $_->{'view'} && $_->{'viewindex'} == $_[1] } @zl;
+	}
+else {
+	@zl = grep { !$_->{'view'} } @zl;
 	}
-@zl = &find("zone", $conf);
 $ipv6 = $config{'support_aaaa'} && &check_ip6address($_[0]);
 if ($ipv6) {
 	@zero = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
@@ -590,9 +620,7 @@ if ($ipv6) {
 		$rev = &net_to_ip6int($addr, 4*($i+1));
 		$rev =~ s/\.$//g;
 		foreach $z (@zl) {
-			if (lc($z->{'value'}) eq $rev &&
-			    ($typed = &find("type", $z->{'members'})) &&
-			    $typed->{'value'} eq "master") {
+			if (lc($z->{'name'}) eq $rev && $z->{'type'} eq 'master') {
 				# found the reverse master domain
 				$revconf = $z;
 				last DOMAIN;
@@ -607,10 +635,9 @@ else {
 			    : &ip_to_arpa(join('.', @octs[0..$i]));
 		$rev =~ s/\.$//g;
 		foreach $z (@zl) {
-			if ((lc($z->{'value'}) eq $rev ||
-			     lc($z->{'value'}) eq "$rev.") &&
-			    ($typed = &find("type", $z->{'members'})) &&
-			    $typed->{'value'} eq "master") {
+			if ((lc($z->{'name'}) eq $rev ||
+			     lc($z->{'name'}) eq "$rev.") &&
+			    $z->{'type'} eq "master") {
 				# found the reverse master domain
 				$revconf = $z;
 				last DOMAIN;
@@ -621,9 +648,8 @@ else {
 
 # find reverse record
 if ($revconf) {
-	$revfile = &absolute_path(
-		&find("file", $revconf->{'members'})->{'value'});
-	@revrecs = &read_zone_file($revfile, $revconf->{'value'});
+	$revfile = &absolute_path($revconf->{'file'});
+	@revrecs = &read_zone_file($revfile, $revconf->{'name'});
 	if ($ipv6) {
 		$addr = &net_to_ip6int($_[0], 128);
 		}
@@ -650,20 +676,21 @@ local ($fwdconf, $i, $fwdfile, $fwdrec, $fr, $ipv6);
 
 # find forward domain
 local $host = $_[0]; $host =~ s/\.$//;
-local $conf = &get_config();
-if ($in{'view'} ne '') {
-	$conf = $conf->[$in{'view'}]->{'members'};
+local @zl = grep { $_->{'type'} ne 'view' } &list_zone_names();
+if ($_[1] ne '') {
+	@zl = grep { $_->{'view'} && $_->{'viewindex'} == $_[1] } @zl;
+	}
+else {
+	@zl = grep { !$_->{'view'} } @zl;
 	}
-local @zl = &find("zone", $conf);
 local @parts = split(/\./, $host);
 DOMAIN: for($i=1; $i<@parts; $i++) {
 	local $fwd = join(".", @parts[$i .. @parts-1]);
 	foreach $z (@zl) {
 		local $typed;
-		if ((lc($z->{'value'}) eq $fwd ||
-		     lc($z->{'value'}) eq "$fwd.") &&
-		    ($typed = &find("type", $z->{'members'})) &&
-		    $typed->{'value'} eq "master") {
+		if ((lc($z->{'name'}) eq $fwd ||
+		     lc($z->{'name'}) eq "$fwd.") &&
+		    $z->{'type'} eq "master") {
 			# Found the forward master!
 			$fwdconf = $z;
 			last DOMAIN;
@@ -673,9 +700,8 @@ DOMAIN: for($i=1; $i<@parts; $i++) {
 
 # find forward record
 if ($fwdconf) {
-	$fwdfile = &absolute_path(
-		&find("file", $fwdconf->{'members'})->{'value'});
-	local @fwdrecs = &read_zone_file($fwdfile, $fwdconf->{'value'});
+	$fwdfile = &absolute_path($fwdconf->{'file'});
+	local @fwdrecs = &read_zone_file($fwdfile, $fwdconf->{'name'});
 	foreach $fr (@fwdrecs) {
 		if ($ipv6 ? $fr->{'type'} eq "AAAA" : $fr->{'type'} eq "A" &&
 		    $fr->{'name'} eq $_[0]) {
@@ -689,39 +715,59 @@ if ($fwdconf) {
 return ($fwdconf, $fwdfile, $fwdrec);
 }
 
-# can_edit_zone(&access, &zone, [&view])
+# can_edit_zone(&zone, [&view] | &cachedzone)
+# Returns 1 if some zone can be edited
 sub can_edit_zone
 {
 local %zcan;
-local $zn = $_[1]->{'value'};
-local $vn = $_[2] ? 'view_'.$_[2]->{'value'} : undef;
+local ($zn, $vn, $file);
+if ($_[0]->{'members'}) {
+	# A full zone structure
+	$zn = $_[0]->{'value'};
+	$vn = $_[1] ? 'view_'.$_[1]->{'value'} : undef;
+	$file = &find_value("file", $_[0]->{'members'});
+	}
+else {
+	# A cached zone object
+	$zn = $_[0]->{'name'};
+	$vn = $_[0]->{'view'} eq '*' ? undef : $_[0]->{'view'};
+	$file = $_[0]->{'file'};
+	}
 
 # Check zone name
-if ($_[0]->{'zones'} eq '*') {
+if ($access{'zones'} eq '*') {
+	# Always can
 	}
-elsif ($_[0]->{'zones'} =~ /^\!/) {
-	foreach (split(/\s+/, $_[0]->{'zones'})) {
+elsif ($access{'zones'} =~ /^\!/) {
+	# List of denied zones
+	foreach (split(/\s+/, $access{'zones'})) {
 		return 0 if ($_ eq $zn || ($vn && $_ eq $vn));
 		}
 	}
 else {
+	# List of allowed zones
 	local $ok;
-	foreach (split(/\s+/, $_[0]->{'zones'})) {
+	foreach (split(/\s+/, $access{'zones'})) {
 		$ok++ if ($_ eq $zn || ($vn && $_ eq $vn));
 		}
 	return 0 if (!$ok);
 	}
 
-if ($_[0]->{'dironly'}) {
+if ($access{'dironly'}) {
 	# Check directory access control 
-	local $filestr = &find("file", $_[1]->{'members'});
-	return 1 if (!$filestr);
-	local $file = &absolute_path($filestr->{'value'});
-	return 0 if (!&allowed_zone_file($_[0], $file));
+	return 1 if (!$file);
+	$file = &absolute_path($file);
+	return 0 if (!&allowed_zone_file(\%access, $file));
 	}
 return 1;
 }
 
+# can_edit_reverse(&zone)
+sub can_edit_reverse
+{
+return $access{'reverse'} || &can_edit_zone($_[0]);
+}
+
 # record_input(zoneindex, view, type, file, origin, [num], [record])
 # Display a form for editing or creating a DNS record
 sub record_input
@@ -929,8 +975,7 @@ elsif ($type eq "PTR") {
 	}
 else { print "<tr> <td colspan=2></td>\n"; }
 print "<td colspan=2 align=right>\n";
-local %a = &get_module_acl();
-if (!$a{'ro'}) {
+if (!$access{'ro'}) {
 	if (@_ >= 6) {
 		print "<input type=submit value=\"$text{'save'}\">\n";
 		print "<input type=submit name=delete ",
@@ -941,14 +986,16 @@ if (!$a{'ro'}) {
 print "</td></tr></table></td></tr></table></form>\n";
 }
 
+# zones_table(&links, &titles, &types, &deletes)
 sub zones_table
 {
 local($i);
 print "<table border width=100%>\n";
-print "<tr $tb> <td><b>$text{'index_zone'}</b></td> ",
+print "<tr $tb> <td width=5><br></td> <td><b>$text{'index_zone'}</b></td> ",
       "<td><b>$text{'index_type'}</b></td> </tr>\n";
 for($i=0; $i<@{$_[0]}; $i++) {
 	print "<tr $cb>\n";
+	print "<td width=5>",&ui_checkbox("d", $_[3]->[$i]),"</td>\n";
 	print "<td><a href=\"$_[0]->[$i]\">$_[1]->[$i]</a></td>\n";
 	print "<td>$_[2]->[$i]</td>\n";
 	print "</tr>\n";
@@ -1108,23 +1155,24 @@ else {
 @severities = ( 'critical', 'error', 'warning', 'notice', 'info',
 		'debug', 'dynamic' );
 
-# can_edit_view(&access, &view)
+# can_edit_view(&view | &viewcache)
+# Returns 1 if some view can be edited
 sub can_edit_view
 {
 local %vcan;
-local $vn = $_[1]->{'value'};
+local $vn = $_[0]->{'members'} ? $_[0]->{'value'} : $_[0]->{'name'};
 
-if ($_[0]->{'vlist'} eq '*') {
+if ($access{'vlist'} eq '*') {
 	return 1;
 	}
-elsif ($_[0]->{'vlist'} =~ /^\!/) {
-	foreach (split(/\s+/, $_[0]->{'vlist'})) {
+elsif ($access{'vlist'} =~ /^\!/) {
+	foreach (split(/\s+/, $access{'vlist'})) {
 		return 0 if ($_ eq $vn);
 		}
 	return 1;
 	}
 else {
-	foreach (split(/\s+/, $_[0]->{'vlist'})) {
+	foreach (split(/\s+/, $access{'vlist'})) {
 		return 1 if ($_ eq $vn);
 		}
 	return 0;
@@ -1196,8 +1244,7 @@ sub make_chroot
 local $chroot = &get_chroot();
 return $_[0] if (!$chroot);
 return $_[0] if ($_[0] eq $config{'named_conf'} && $config{'no_chroot'});
-if ($config{'no_pid_chroot'} &&
-    ($_[1] || $_[0] eq &get_pid_file())) {
+if ($config{'no_pid_chroot'} && $_[1]) {
 	return $_[0];
 	}
 return $chroot.$_[0];
@@ -1216,38 +1263,48 @@ if ($config{'ndc_cmd'} =~ /^(\S+)/ && &has_command("$1") && $_[0] != 1) {
 return 0;
 }
 
-# get_pid_file()
+# get_pid_file([no-cache])
 # Returns the BIND pid file path, relative to any chroot
 sub get_pid_file
 {
-local $conf = &get_config();
-local ($opts, $pidopt);
-if (($opts = &find("options", $conf)) &&
-    ($pidopt = &find("pid-file", $opts->{'members'}))) {
-	# read from PID file
-	local $pidfile = $pidopt->{'value'};
-	if ($pidfile !~ /^\//) {
-		local $dir = &find("directory", $opts->{'members'});
-		$pidfile = $dir->{'value'}."/".$pidfile;
+if ($_[0]) {
+	# Read real config
+	local $conf = &get_config();
+	local ($opts, $pidopt);
+	if (($opts = &find("options", $conf)) &&
+	    ($pidopt = &find("pid-file", $opts->{'members'}))) {
+		# read from PID file
+		local $pidfile = $pidopt->{'value'};
+		if ($pidfile !~ /^\//) {
+			local $dir = &find("directory", $opts->{'members'});
+			$pidfile = $dir->{'value'}."/".$pidfile;
+			}
+		return $pidfile;
 		}
-	return $pidfile;
-	}
 
-# use default file
-foreach $p (split(/\s+/, $config{'pid_file'})) {
-	if (-r &make_chroot($p, 1)) {
-		return $p;
+	# use default file
+	local $p;
+	foreach $p (split(/\s+/, $config{'pid_file'})) {
+		if (-r &make_chroot($p, 1)) {
+			return $p;
+			}
 		}
+	return "/var/run/named.pid";
+	}
+else {
+	# Use cache if possible
+	local %znc;
+	&read_file_cached($zone_names_cache, \%znc);
+	return $znc{'pidfile'} || &get_pid_file(1);
 	}
-return "/var/run/named.pid";
 }
 
-# can_edit_type(record-type, &access)
+# can_edit_type(record-type)
 sub can_edit_type
 {
-return 1 if (!$_[1]->{'types'});
+return 1 if (!$access{'types'});
 local $t;
-foreach $t (split(/\s+/, $_[1]->{'types'})) {
+foreach $t (split(/\s+/, $access{'types'})) {
 	return 1 if (lc($t) eq lc($_[0]));
 	}
 return 0;
@@ -1277,7 +1334,7 @@ local $form = defined($_[2]) ? $_[2] : 0;
 return "<input type=button onClick='ifield = document.forms[$form].$_[0]; chooser = window.open(\"free_chooser.cgi\", \"chooser\", \"toolbar=no,menubar=no,scrollbars=yes,width=150,height=500\"); chooser.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
 }
 
-# create_slave_zone(name, master-ip, [view])
+# create_slave_zone(name, master-ip, [view], [file])
 # A convenience function for creating a new slave zone, if it doesn't exist
 # yet. Mainly useful for Virtualmin, to avoid excessive transfer of BIND
 # configuration data.
@@ -1292,11 +1349,13 @@ if (!$opts) {
 	return 1;
 	}
 
-# Check if exists
-local @zones = &find("zone", $conf);
-local $v;
-foreach $v (&find("view", $conf)) {
-	push(@zones, &find("zone", $v->{'members'}));
+# Check if exists in the view
+if ($_[2]) {
+	local ($v) = grep { $_->{'value'} eq $_[2] } &find("view", $conf);
+	@zones = &find("zone", $v->{'members'});
+	}
+else {
+	@zones = &find("zone", $conf);
 	}
 local ($z) = grep { $_->{'value'} eq $_[0] } @zones;
 return 2 if ($z);
@@ -1314,14 +1373,31 @@ local $dir = { 'name' => 'zone',
                             ]
 	     };
 local $base = $config{'slave_dir'} || &base_directory();
-local $file = $base."/".$_[0].".hosts";
-push(@{$dir->{'members'}}, { 'name' => 'file',
-			     'values' => [ $file ] } );
+if (!$_[3]) {
+	# File has default name and is under default directory
+	local $file;
+	if ($_[0] =~ /in-addr/i) {
+		local $dom = &ip6int_to_net(&arpa_to_ip($_[0]));
+		$dom = s/\//_/;
+		$file = $sbase."/".$dom.".rev";
+		}
+	else {
+		$file = $base."/".$_[0].".hosts";
+		}
+	push(@{$dir->{'members'}}, { 'name' => 'file',
+				     'values' => [ $file ] } );
+	}
+elsif ($_[3] ne "none") {
+	# File was specified
+	local $file = $_[3] =~ /^\// ? $_[3] : $base."/".$_[3];
+	push(@{$dir->{'members'}}, { 'name' => 'file',
+				     'values' => [ $file ] } );
+	}
 if ($_[2]) {
 	# Create in a view
-	local $view = grep { $_->{'value'} eq $_[2] } &find("view", $conf);
+	local ($view) = grep { $_->{'value'} eq $_[2] } &find("view", $conf);
 	return 3 if (!$view);
-	&save_directive($view, undef, [ $dir ], 0);
+	&save_directive($view, undef, [ $dir ], 1);
 	}
 else {
 	# Create at top level
@@ -1334,7 +1410,7 @@ return 0;
 
 # delete_zone(name, [view])
 # Delete one zone from named.conf
-# Returns 0 on success, or 1 if the zone was not found
+# Returns 0 on success, 1 if the zone was not found, or 2 if the view was not found.
 sub delete_zone
 {
 local $parent = &get_config_parent();
@@ -1343,7 +1419,9 @@ local @zones;
 if ($_[1]) {
 	# Look in one view
 	local ($v) = grep { $_->{'value'} eq $_[1] } &find("view", $conf);
-	local @zones = &find("zone", $v->{'members'});
+	return 2 if (!$v);
+	@zones = &find("zone", $v->{'members'});
+	$parent = $v;
 	}
 else {
 	# Look in all views
@@ -1358,7 +1436,51 @@ return 1 if (!$z);
 
 &save_directive($parent, [ $z ], [ ]);
 &flush_file_lines();
+return 0;
+}
 
+# rename_zone(oldname, newname, [view])
+# Changes the name of some zone, and perhaps it's file
+# Returns 0 on success, 1 if the zone was not found, or 2 if the view was
+# not found.
+sub rename_zone
+{
+local $parent = &get_config_parent();
+local $conf = $parent->{'members'};
+local @zones;
+if ($_[2]) {
+	# Look in one view
+	local ($v) = grep { $_->{'value'} eq $_[2] } &find("view", $conf);
+	return 2 if (!$v);
+	@zones = &find("zone", $v->{'members'});
+	$parent = $v;
+	}
+else {
+	# Look in all views
+	@zones = &find("zone", $conf);
+	local $v;
+	foreach $v (&find("view", $conf)) {
+		push(@zones, &find("zone", $v->{'members'}));
+		}
+	}
+local ($z) = grep { $_->{'value'} eq $_[0] } @zones;
+return 1 if (!$z);
+
+$z->{'values'} = [ $_[1] ];
+local $file = &find("file", $z->{'members'});
+if ($file) {
+	# Update the file too
+	local $newfile = $file->{'values'}->[0];
+	$newfile =~ s/$_[0]/$_[1]/g;
+	if ($newfile ne $file->{'values'}->[0]) {
+		rename(&make_chroot($file->{'values'}->[0]),
+		       &make_chroot($newfile));
+		$file->{'values'}->[0] = $newfile;
+		}
+	}
+
+&save_directive($parent, [ $z ], [ $z ]);
+&flush_file_lines();
 return 0;
 }
 
@@ -1375,10 +1497,7 @@ if ($config{'restart_cmd'}) {
 	}
 else {
 	local $pidfile = &get_pid_file();
-	local $pid;
-	open(PIDFILE, &make_chroot($pidfile));
-	chop($pid = <PIDFILE>);
-	close(PIDFILE);
+	local $pid = &check_pid_file(&make_chroot($pidfile, 1));
 	if (!$pid) {
 		return &text('restart_epidfile', $pidfile);
 		}
@@ -1389,6 +1508,73 @@ else {
 return undef;
 }
 
+# start_bind()
+# Attempts to start the BIND DNS server, and returns undef on success or an
+# error message on failure
+sub start_bind
+{
+local $chroot = &get_chroot();
+local $user;
+local $cmd;
+if ($config{'named_user'}) {
+	$user = "-u $config{'named_user'}";
+	if ($config{'named_group'}) {
+		$user .= " -g $config{'named_group'}";
+		}
+	else {
+		local @u = getpwnam($config{'named_user'});
+		local @g = getgrgid($u[3]);
+		$user .= " -g $g[0]";
+		}
+	}
+if ($config{'start_cmd'}) {
+	$cmd = $config{'start_cmd'};
+	}
+elsif (!$chroot) {
+	$cmd = "$config{'named_path'} -c $config{'named_conf'} $user </dev/null 2>&1";
+	}
+elsif (`$config{'named_path'} -help 2>&1` =~ /\[-t/) {
+	# use named's chroot option
+	$cmd = "$config{'named_path'} -c $config{'named_conf'} -t $chroot $user </dev/null 2>&1";
+	}
+else {
+	# use the chroot command
+	$cmd = "chroot $chroot $config{'named_path'} -c $config{'named_conf'} $user </dev/null 2>&1";
+	}
+
+local $temp = &transname();
+local $rv = &system_logged("$cmd </dev/null >$temp 2>&1");
+local $out = `cat $temp`;
+unlink($temp);
+if ($rv || $out =~ /chroot.*not available/i) {
+	return &text('start_error', $out ? "<tt>$out</tt>" : "Unknown error");
+	}
+return undef;
+}
+
+# stop_bind()
+# Kills the running DNS server, and returns undef on success or an error message
+# upon failure
+sub stop_bind
+{
+if ($config{'stop_cmd'}) {
+	# Just use a command
+	local $out = &backquote_logged("($config{'stop_cmd'}) 2>&1");
+	if ($?) {
+		return "<pre>$out</pre>";
+		}
+	}
+else {
+	# Kill the process
+	local $pidfile = &get_pid_file();
+	local $pid = &check_pid_file(&make_chroot($pidfile, 1));
+	if (!$pid || !&kill_logged('TERM', $pid)) {
+		return $text{'stop_epid'};
+		}
+	}
+return undef;
+}
+
 # version_atleast(v1, v2, v3)
 sub version_atleast
 {
@@ -1417,5 +1603,488 @@ foreach $c (@$vconf) {
 return undef;
 }
 
+# create_zone(&zone, &conf, [view-idx])
+# Convenience function for adding a new zone
+sub create_zone
+{
+local ($dir, $conf, $viewidx) = @_;
+if ($viewidx ne "") {
+	# Adding inside a view
+	local $view = $conf->[$in{'view'}];
+        &lock_file(&make_chroot($view->{'file'}));
+        &save_directive($view, undef, [ $dir ], 1);
+        &flush_file_lines();
+        &unlock_file(&make_chroot($view->{'file'}));
+	}
+else {
+	# Adding at top level
+        $dir->{'file'} = &add_to_file();
+        local $pconf = &get_config_parent($dir->{'file'});
+        &lock_file(&make_chroot($dir->{'file'}));
+        &save_directive($pconf, undef, [ $dir ], 0);
+        &flush_file_lines();
+        &unlock_file(&make_chroot($dir->{'file'}));
+	}
+}
+
+$heiropen_file = "$module_config_directory/heiropen";
+
+# get_heiropen()
+# Returns an array of open categories
+sub get_heiropen
+{
+open(HEIROPEN, $heiropen_file);
+local @heiropen = <HEIROPEN>;
+chop(@heiropen);
+close(HEIROPEN);
+return @heiropen;
+}
+
+# save_heiropen(&heir)
+sub save_heiropen
+{
+&open_tempfile(HEIR, ">$heiropen_file");
+foreach $h (@{$_[0]}) {
+	&print_tempfile(HEIR, $h,"\n");
+	}
+&close_tempfile(HEIR);
+}
+
+# list_zone_names()
+# Returns a list of zone names, types, files and views based on a cache
+# built from the primary configuration.
+sub list_zone_names
+{
+local @st = stat($zone_names_cache);
+local %znc;
+&read_file_cached($zone_names_cache, \%znc);
+
+# Check if any files have changed
+local @files;
+local ($k, $changed, $filecount);
+foreach $k (keys %znc) {
+	if ($k =~ /^file_(.*)$/) {
+		$filecount++;
+		local @fst = stat($1);
+		if ($fst[9] > $st[9]) {
+			$changed = 1;
+			}
+		}
+	}
+if ($changed || !$filecount || $znc{'version'} != $zone_names_version) {
+	# Yes .. need to rebuild
+	%znc = ( );
+	local $conf = &get_config();
+	local @views = &find("view", $conf);
+	local ($v, $z);
+	local $n = 0;
+	foreach $v (@views) {
+		local @vz = &find("zone", $v->{'members'});
+		foreach $z (@vz) {
+			local $type = &find_value("type", $z->{'members'});
+			local $file = &find_value("file", $z->{'members'});
+			$znc{"zone_".($n++)} = "$z->{'value'} $z->{'index'} $type $v->{'value'} $file";
+			$files{$z->{'file'}}++;
+			}
+		$znc{"view_".($n++)} = "$v->{'value'} $v->{'index'}";
+		$files{$v->{'file'}}++;
+		}
+	foreach $z (&find("zone", $conf)) {
+		local $type = &find_value("type", $z->{'members'});
+		local $file = &find_value("file", $z->{'members'});
+		$znc{"zone_".($n++)} = "$z->{'value'} $z->{'index'} $type * $file";
+		$files{$z->{'file'}}++;
+		}
+
+	# Store the base directory and PID file
+	$znc{'base'} = &base_directory($conf, 1);
+	$znc{'pidfile'} = &get_pid_file(1);
+
+	# Store source files
+	foreach $f (keys %files) {
+		local $realf = &make_chroot(&absolute_path($f));
+		local @st = stat($realf);
+		$znc{"file_".$realf} = $st[9];
+		}
+
+	$znc{'version'} = $zone_names_version;
+	&write_file($zone_names_cache, \%znc);
+	undef(@list_zone_names_cache);
+	}
+
+# Use in-memory cache
+if (defined(@list_zone_names_cache)) {
+	return @list_zone_names_cache;
+	}
+
+# Construct the return value from the hash
+local (@rv, %viewidx);
+foreach $k (keys %znc) {
+	if ($k =~ /^zone_(\d+)$/) {
+		local ($name, $index, $type, $view, $file) =
+			split(/\s+/, $znc{$k}, 5);
+		push(@rv, { 'name' => $name,
+			    'type' => $type,
+			    'index' => $index,
+			    'view' => $view eq '*' ? undef : $view,
+			    'file' => $file });
+		}
+	elsif ($k =~ /^view_(\d+)$/) {
+		local ($name, $index) = split(/\s+/, $znc{$k}, 2);
+		push(@rv, { 'name' => $name,
+			    'index' => $index,
+			    'type' => 'view' });
+		$viewidx{$name} = $index;
+		}
+	}
+local $z;
+foreach $z (@rv) {
+	if ($z->{'type'} ne 'view' && $z->{'view'} ne '*') {
+		$z->{'viewindex'} = $viewidx{$z->{'view'}};
+		}
+	}
+@list_zone_names_cache = @rv;
+return @rv;
+}
+
+# get_zone_name(index|name, [viewindex])
+# Returns a zone cache object, looked up by name or index
+sub get_zone_name
+{
+local @zones = &list_zone_names();
+local $field = $_[0] =~ /^\d+$/ ? "index" : "name";
+local $z;
+foreach $z (@zones) {
+	if ($z->{$field} eq $_[0] &&
+	    ($_[1] eq '' && !defined($z->{'viewindex'}) ||
+	     $_[1] ne '' && $z->{'viewindex'} == $_[1])) {
+		return $z;
+		}
+	}
+return undef;
+}
+
+# list_slave_servers()
+# Returns a list of Webmin servers on which slave zones are created / deleted
+sub list_slave_servers
+{
+&foreign_require("servers", "servers-lib.pl");
+local %ids = map { $_, 1 } split(/\s+/, $config{'servers'});
+local @servers = &servers::list_servers();
+if (%ids) {
+	return grep { $ids{$_->{'id'}} } @servers;
+	}
+elsif ($config{'default_slave'} && !defined($config{'servers'})) {
+	# Migrate old-style setting of single slave
+	local ($serv) = grep { $_->{'host'} eq $config{'default_slave'} }
+			     @servers;
+	if ($serv) {
+		&add_slave_server($serv);
+		return ($serv);
+		}
+	}
+return ( );
+}
+
+# add_slave_server(&server)
+sub add_slave_server
+{
+&lock_file($module_config);
+&foreign_require("servers", "servers-lib.pl");
+local @sids = split(/\s+/, $config{'servers'});
+$config{'servers'} = join(" ", @sids, $_[0]->{'id'});
+&sync_default_slave();
+&save_module_config();
+&unlock_file($module_config);
+&servers::save_server($_[0]);
+}
+
+# delete_slave_server(&server)
+sub delete_slave_server
+{
+&lock_file($module_config);
+local @sids = split(/\s+/, $config{'servers'});
+$config{'servers'} = join(" ", grep { $_ != $_[0]->{'id'} } @sids);
+&sync_default_slave();
+&save_module_config();
+&unlock_file($module_config);
+}
+
+sub sync_default_slave
+{
+local @servers = &list_slave_servers();
+if (@servers) {
+	$config{'default_slave'} = $servers[0]->{'host'};
+	}
+else {
+	$config{'default_slave'} = '';
+	}
+}
+
+# server_name(&server)
+sub server_name
+{
+return $_[0]->{'desc'} ? $_[0]->{'desc'} : $_[0]->{'host'};
+}
+
+# create_master_records(file, zone, master, email, refresh, retry, expiry, min,
+#			add-master-ns, add-slaves-ns, add-template, tmpl-ip)
+# Creates the records file for a new master zone. Returns undef on success, or
+# an error message on failure.
+sub create_master_records
+{
+local ($file, $zone, $master, $email, $refresh, $retry, $expiry, $min,
+       $add_master, $add_slaves, $add_tmpl, $ip) = @_;
+
+# Create the zone file
+&lock_file(&make_chroot($file));
+&open_tempfile(ZONE, ">".&make_chroot($file), 1) ||
+	return &text('create_efile3', $file, $!);
+&print_tempfile(ZONE, "\$ttl $min\n")
+	if ($config{'master_ttl'});
+&close_tempfile(ZONE);
+
+# create the SOA and NS records
+local $serial;
+if ($config{'soa_style'} == 1) {
+        $serial = &date_serial().sprintf("%2.2d", $config{'soa_start'});
+        }
+else {
+        $serial = time();
+        }
+local $vals = "$master $email (\n".
+        "\t\t\t$serial\n".
+        "\t\t\t$refresh\n".
+        "\t\t\t$retry\n".
+        "\t\t\t$expiry\n".
+        "\t\t\t$min )";
+&create_record($file, "$zone.", undef, "IN", "SOA", $vals);
+&create_record($file, "$zone.", undef, "IN", "NS", $master)
+	if ($add_master);
+if ($add_slave) {
+	local $slave;
+	foreach $slave (&list_slave_servers()) {
+		local @bn = gethostbyname($slave->{'host'});
+		local $full = "$bn[0].";
+		&create_record($file, "$zone.", undef, "IN", "NS", $full);
+		}
+	}
+
+if ($add_tmpl) {
+	# Create template records
+	local $i;
+	for($i=0; $config{"tmpl_$i"}; $i++) {
+		local @c = split(/\s+/, $config{"tmpl_$i"}, 3);
+		&create_record($file,
+			       $c[0] eq '.' ? "$zone." : $c[0], undef,
+			       "IN", $c[1], $c[2] ? $c[2] : $ip);
+		}
+	if ($config{'tmpl_include'}) {
+		# Add whatever is in the template file
+		open(TMPL, $config{'tmpl_include'});
+		&open_tempfile(FILE, ">>".&make_chroot($file));
+		while(<TMPL>) {
+			&print_tempfile(FILE, $_);
+			}
+		close(TMPL);
+		&close_tempfile(FILE);
+		}
+	}
+&unlock_file(&make_chroot($file));
+&set_ownership(&make_chroot(file));
+return undef;
+}
+
+# automatic_filename(domain, is-reverse, base)
+# Returns a filename for a new zone
+sub automatic_filename
+{
+local ($zone, $rev, $base) = @_;
+local $file;
+if ($rev) {
+	# create filename for reverse zone
+	$file = &ip6int_to_net(&arpa_to_ip($zone));
+	$file =~ s/\//_/;
+	local $format = $config{'reversezonefilename_format'};
+	$format =~ s/ZONE/$file/g;
+	$file = $base."/".$format;
+	}
+else {
+	# create filename for forward zone
+	local $format = $config{'forwardzonefilename_format'};
+	$format =~ s/ZONE/$zone/g;
+	$file = $base."/".$format;
+	}
+return $file;
+}
+
+# create_on_slaves(zone, master-ip, file, [&hostnames])
+# Creates the given zone on all configured slave servers, and returns a list
+# of errors
+sub create_on_slaves
+{
+local ($zone, $master, $file, $hosts) = @_;
+local %on = map { $_, 1 } @$hosts;
+&remote_error_setup(\&slave_error_handler);
+local $slave;
+local @slaveerrs;
+foreach $slave (&list_slave_servers()) {
+	next if (%on && !$on{$slave->{'host'}});
+
+	# Connect to server
+	$slave_error = undef;
+	&remote_foreign_require($slave, "bind8", "bind8-lib.pl");
+	if ($slave_error) {
+		push(@slaveerrs, [ $slave, $slave_error ]);
+		next;
+		}
+
+	# Create the zone
+	local $err = &remote_foreign_call($slave, "bind8",
+		"create_slave_zone", $zone, $master,
+		$slave->{'bind8_view'}, $file);
+	if ($err == 1) {
+		push(@slaveerrs, [ $slave, $text{'master_esetup'} ]);
+		}
+	elsif ($err == 2) {
+		push(@slaveerrs, [ $slave, $text{'master_etaken'} ]);
+		}
+	elsif ($err == 3) {
+		push(@slaveerrs, [ $slave, &text('master_eview',
+					 $slave->{'bind8_view'}) ]);
+		}
+	}
+&remote_error_setup();
+return @slaveerrs;
+}
+
+# delete_on_slaves(domain, [&slave-hostnames])
+# Delete some domain or all or listed slave servers
+sub delete_on_slaves
+{
+local $dom = $_[0];
+local %on = map { $_, 1 } @{$_[1]};
+&remote_error_setup(\&slave_error_handler);
+local $slave;
+local @slaveerrs;
+foreach $slave (&list_slave_servers()) {
+	next if (%on && !$on{$slave->{'host'}});
+
+	# Connect to server
+	$slave_error = undef;
+	&remote_foreign_require($slave, "bind8", "bind8-lib.pl");
+	if ($slave_error) {
+		push(@slaveerrs, [ $slave, $slave_error ]);
+		next;
+		}
+
+	# Delete the zone
+	$err = &remote_foreign_call($slave, "bind8", "delete_zone",
+			    $dom, $slave->{'bind8_view'});
+	if ($err == 1) {
+		push(@slaveerrs, [ $slave, $text{'delete_ezone'} ]);
+		}
+	elsif ($err == 2) {
+		push(@slaveerrs, [ $slave, &text('master_eview',
+					 $slave->{'bind8_view'}) ]);
+		}
+	}
+&remote_error_setup();
+return @slaveerrs;
+}
+
+# rename_on_slaves(olddomain, newdomain, [&slave-hostnames])
+# Changes the name of some domain on all or listed slave servers
+sub rename_on_slaves
+{
+local ($olddom, $newdom, $on) = @_;
+local %on = map { $_, 1 } @$on;
+&remote_error_setup(\&slave_error_handler);
+local $slave;
+local @slaveerrs;
+foreach $slave (&list_slave_servers()) {
+	next if (%on && !$on{$slave->{'host'}});
+
+	# Connect to server
+	$slave_error = undef;
+	&remote_foreign_require($slave, "bind8", "bind8-lib.pl");
+	if ($slave_error) {
+		push(@slaveerrs, [ $slave, $slave_error ]);
+		next;
+		}
+
+	# Delete the zone
+	$err = &remote_foreign_call($slave, "bind8", "rename_zone",
+			    $olddom, $newdom, $slave->{'bind8_view'});
+	if ($err == 1) {
+		push(@slaveerrs, [ $slave, $text{'delete_ezone'} ]);
+		}
+	elsif ($err == 2) {
+		push(@slaveerrs, [ $slave, &text('master_eview',
+					 $slave->{'bind8_view'}) ]);
+		}
+	}
+&remote_error_setup();
+return @slaveerrs;
+}
+
+# restart_on_slaves([&slave-hostnames])
+# Re-starts BIND on all or listed slave servers, and returns a list of errors
+sub restart_on_slaves
+{
+local %on = map { $_, 1 } @{$_[0]};
+&remote_error_setup(\&slave_error_handler);
+local $slave;
+local @slaveerrs;
+foreach $slave (&list_slave_servers()) {
+	next if (%on && !$on{$slave->{'host'}});
+
+	# Find the PID file
+	$slave_error = undef;
+	&remote_foreign_require($slave, "bind8", "bind8-lib.pl");
+	if ($slave_error) {
+		push(@slaveerrs, [ $slave, $slave_error ]);
+		next;
+		}
+	local $sver = &remote_foreign_call($slave, "bind8",
+				     "get_webmin_version");
+	local $pidfile;
+	if ($sver >= 1.140) {
+		# Call new function to get PID file from slave
+		$pidfile = &remote_foreign_call(
+			$slave, "bind8", "get_pid_file");
+		$pidfile = &remote_foreign_call(
+			$slave, "bind8", "make_chroot", $pidfile, 1);
+		}
+	else {
+		push(@slaveerrs, [ $slave, &text('restart_eversion',
+						 $slave->{'host'}, 1.140) ]);
+		next;
+		}
+
+	# Read the PID and restart
+	local $pid = &remote_foreign_call($slave, "bind8",
+				    "check_pid_file", $pidfile);
+	if (!$pid) {
+		push(@slaveerrs, [ $slave, &text('restart_erunning2',
+						 $slave->{'host'}) ]);
+		next;
+		}
+	$err = &remote_foreign_call($slave, "bind8", "restart_bind");
+	if ($err) {
+		push(@slaveerrs, [ $slave, &text('restart_esig2',
+						 $slave->{'host'}, $err) ]);
+		}
+	}
+&remote_error_setup();
+return @slaveerrs;
+}
+
+sub slave_error_handler
+{
+$slave_error = $_[0];
+}
+
+
 1;
 
