@@ -327,10 +327,15 @@ for($i=0; $i<@{$_[1]}; $i++) {
 		# already set serial if no acl allow it to update or update is disabled
 		$serial = $v->[2];
 		if ($config{'updserial_on'}) { # generally handle serial numbers ?
-			if ($config{'soa_style'} == 1 && $v->[2] =~ /^(\d{8})(\d\d)$/) {
-				if ($1 eq &date_serial())
-					{ $serial = sprintf "%d%2.2d", $1, $2+1; }
-				else { $serial = &date_serial()."00"; }
+			if ($config{'soa_style'} == 1 &&
+			    $v->[2] =~ /^(\d{8})(\d\d)$/) {
+				if ($1 eq &date_serial()) {
+					$serial = sprintf "%d%2.2d", $1, $2+1;
+					}
+				else {
+					$serial = &date_serial().
+					 sprintf("%2.2d", $config{'soa_start'});
+					}
 				}
 			else { $serial = $v->[2]+1; }
 		}
