@@ -11,11 +11,12 @@ sub check_iptables
 if (!-r $init_script) {
 	return &text('redhat_escript', "<tt>$init_script</tt>");
 	}
-if ($gconfig{'os_type'} ne 'trustix-linux') {
-	local $out = `$init_script status 2>&1`;
-	if ($out !~ /table:|INPUT|FORWARD|OUTPUT|is\s+stopped|firewall\s+stopped/) {
-		return &text('redhat_eoutput', "<tt>$init_script status</tt>");
-		}
+return undef if ($gconfig{'os_type'} eq 'trustix-linux');
+return undef if ($gconfig{'os_type'} eq 'redhat-linux' &&
+		 $gconfig{'os_version'} > 10);
+local $out = `$init_script status 2>&1`;
+if ($out !~ /table:|INPUT|FORWARD|OUTPUT|is\s+stopped|firewall\s+stopped/) {
+	return &text('redhat_eoutput', "<tt>$init_script status</tt>");
 	}
 return undef;
 }
@@ -37,6 +38,10 @@ sub unapply_iptables
 {
 $out = &backquote_logged("cd / ; $init_script save 2>&1 </dev/null");
 $out =~ s/\033[^m]+m//g;
+if ($? && $out =~ /usage/i) {
+	# 'save' argument not supported .. call iptables-save manually
+	return &iptables_save();
+	}
 return $? || $out =~ /FAILED/ ? "<pre>$out</pre>" : undef;
 }
 
