@@ -10,20 +10,6 @@ if ($lib) {
 	do $lib;
 	}
 
-# delete_service(line)
-sub delete_service
-{
-local(@serv);
-open(SERVICES, $config{services_file});
-@serv = <SERVICES>;
-close(SERVICES);
-splice(@serv, $_[0], 1);
-open(SERVICES, "> $config{services_file}");
-print SERVICES @serv;
-close(SERVICES);
-system("$config{'put_services_command'}") if ($config{'put_services_command'});
-}
-
 # list_inets_files()
 # Returns a list of inetd configuration files
 sub list_inets_files
@@ -64,9 +50,9 @@ return @rv;
 # Create a new rpc file entry
 sub create_rpc
 {
-open(RPC, ">> $config{rpc_file}");
-print RPC "$_[0]\t$_[1]",($_[2] ? "\t$_[2]\n" : "\n");
-close(RPC);
+&open_tempfile(RPC, ">> $config{rpc_file}");
+&print_tempfile(RPC, "$_[0]\t$_[1]",($_[2] ? "\t$_[2]\n" : "\n"));
+&close_tempfile(RPC);
 }
 
 
@@ -79,9 +65,9 @@ open(RPC, $config{rpc_file});
 @rpcs = <RPC>;
 close(RPC);
 $rpcs[$_[0]] = "$_[1]\t$_[2]".($_[3] ? "\t$_[3]\n" : "\n");
-open(RPC, "> $config{rpc_file}");
-print RPC @rpcs;
-close(RPC);
+&open_tempfile(RPC, "> $config{rpc_file}");
+&print_tempfile(RPC, @rpcs);
+&close_tempfile(RPC);
 }
 
 
@@ -94,9 +80,9 @@ open(RPC, $config{rpc_file});
 @rpcs = <RPC>;
 close(RPC);
 splice(@rpcs, $_[0], 1);
-open(RPC, "> $config{rpc_file}");
-print RPC @rpcs;
-close(RPC);
+&open_tempfile(RPC, "> $config{rpc_file}");
+&print_tempfile(RPC, @rpcs);
+&close_tempfile(RPC);
 }
 
 
