@@ -46,20 +46,20 @@ return $nis;
 sub save_client_config
 {
 # Save the config file
-open(CONF, ">$config{'client_conf'}");
-open(DOMAIN, ">/etc/defaultdomain");
+&open_tempfile(CONF, ">$config{'client_conf'}");
+&open_tempfile(DOMAIN, ">/etc/defaultdomain");
 if ($_[0]->{'domain'}) {
 	if (!$_[0]->{'broadcast'}) {
 		local @s = @{$_[0]->{'servers'}};
-		print CONF "domain $_[0]->{'domain'} server ",shift(@s),"\n";
+		&print_tempfile(CONF, "domain $_[0]->{'domain'} server ",shift(@s),"\n");
 		foreach $s (@s) {
-			print CONF "ypserver $s\n";
+			&print_tempfile(CONF, "ypserver $s\n");
 			}
 		}
-	print DOMAIN $_[0]->{'domain'},"\n";
+	&print_tempfile(DOMAIN, $_[0]->{'domain'},"\n");
 	}
-close(CONF);
-close(DOMAIN);
+&close_tempfile(CONF);
+&close_tempfile(DOMAIN);
 if ($_[0]->{'domain'}) {
 	&init::enable_at_boot("nis");
 	}
@@ -214,11 +214,11 @@ foreach $t (@nis_files) {
 		}
 	}
 
-open(SLAVES, ">/var/yp/ypservers");
+&open_tempfile(SLAVES, ">/var/yp/ypservers");
 foreach $s (split(/\s+/, $in{'slaves'})) {
-	print SLAVES "$s\n";
+	&print_tempfile(SLAVES, "$s\n");
 	}
-close(SLAVES);
+&close_tempfile(SLAVES);
 
 if ($in{'boot'}) {
 	&init::enable_at_boot("nis");
