@@ -46,20 +46,20 @@ return $nis;
 sub save_client_config
 {
 # Save the config file
-open(CONF, ">$config{'client_conf'}");
+&open_tempfile(CONF, ">$config{'client_conf'}");
 if ($_[0]->{'domain'}) {
 	if ($_[0]->{'broadcast'}) {
-		print CONF "domain $_[0]->{'domain'} broadcast\n";
+		&print_tempfile(CONF, "domain $_[0]->{'domain'} broadcast\n");
 		}
 	else {
 		local @s = @{$_[0]->{'servers'}};
-		print CONF "domain $_[0]->{'domain'} server ",shift(@s),"\n";
+		&print_tempfile(CONF, "domain $_[0]->{'domain'} server ",shift(@s),"\n");
 		foreach $s (@s) {
-			print CONF "ypserver $s\n";
+			&print_tempfile(CONF, "ypserver $s\n");
 			}
 		}
 	}
-close(CONF);
+&close_tempfile(CONF);
 if ($_[0]->{'domain'}) {
 	&init::enable_at_boot("nis-client");
 	}
