@@ -49,24 +49,24 @@ return $nis;
 sub save_client_config
 {
 # Save the config file
-open(CONF, ">$config{'client_conf'}");
+&open_tempfile(CONF, ">$config{'client_conf'}");
 if ($_[0]->{'domain'}) {
 	if ($_[0]->{'broadcast'}) {
-		print CONF "domain $_[0]->{'domain'} broadcast\n";
+		&print_tempfile(CONF, "domain $_[0]->{'domain'} broadcast\n");
 		}
 	else {
 		local @s = @{$_[0]->{'servers'}};
-		print CONF "domain $_[0]->{'domain'} server $s[0]\n";
+		&print_tempfile(CONF,"domain $_[0]->{'domain'} server $s[0]\n");
 		foreach $s (@s) {
-			print CONF "ypserver $s\n";
+			&print_tempfile(CONF, "ypserver $s\n");
 			}
 		}
 	}
-close(CONF);
+&close_tempfile(CONF);
 if ($_[0]->{'domain'}) {
-	open(DOMAIN, ">$default_domain");
-	print DOMAIN $_[0]->{'domain'},"\n";
-	close(DOMAIN);
+	&open_tempfile(DOMAIN, ">$default_domain");
+	&print_tempfile(DOMAIN, $_[0]->{'domain'},"\n");
+	&close_tempfile(DOMAIN);
 	}
 else {
 	unlink($default_domain);
@@ -210,11 +210,11 @@ foreach $t (@nis_files) {
 		}
 	}
 
-open(SLAVES, ">/var/yp/ypservers");
+&open_tempfile(SLAVES, ">/var/yp/ypservers");
 foreach $s (split(/\s+/, $in{'slaves'})) {
-	print SLAVES "$s\n";
+	&print_tempfile(SLAVES, "$s\n");
 	}
-close(SLAVES);
+&close_tempfile(SLAVES);
 
 local @pid = &find_byname("ypserv");
 if ($in{'boot'}) {
