@@ -55,14 +55,16 @@ if ($_[0]->{'domain'}) {
 		}
 
 	# Write the files
-	open(DOM, ">/etc/defaultdomain");
-	print DOM $_[0]->{'domain'},"\n";
-	close(DOM);
+	&open_tempfile(DOM, ">/etc/defaultdomain");
+	&print_tempfile(DOM, $_[0]->{'domain'},"\n");
+	&close_tempfile(DOM);
 	mkdir("$binding_dir/$_[0]->{'domain'}", 0755);
 	if (@s) {
-		open(SRV, ">$binding_dir/$_[0]->{'domain'}/ypservers");
-		foreach $s (@s) { print SRV "$s\n"; }
-		close(SRV);
+		&open_tempfile(SRV,">$binding_dir/$_[0]->{'domain'}/ypservers");
+		foreach $s (@s) {
+			&print_tempfile(SRV, "$s\n");
+			}
+		&close_tempfile(SRV);
 		}
 	else {
 		unlink("$binding_dir/$_[0]->{'domain'}/ypservers");
@@ -187,9 +189,9 @@ if ($in{'domain_def'}) {
 	}
 else {
 	local $old = `domainname`; chop($old);
-	open(DOM, ">/etc/defaultdomain");
-	print DOM "$in{'domain'}\n";
-	close(DOM);
+	&open_tempfile(DOM, ">/etc/defaultdomain");
+	&print_tempfile(DOM, "$in{'domain'}\n");
+	&close_tempfile(DOM);
 	&system_logged("domainname \"$in{'domain'}\" >/dev/null 2>&1");
 	if ($in{'boot'}) {
 		# Create the domain directory
@@ -205,7 +207,7 @@ if ($in{'type'}) {
 		if (!$in{'domain_def'} && -d "/var/yp/$in{'domain'}");
 	}
 else {
-	local $temp = &tempname();
+	local $temp = &transname();
 	open(TEMP, ">$temp");
 	print TEMP "n\ny\n";
 	close(TEMP);
