@@ -41,20 +41,20 @@ return $nis;
 sub save_client_config
 {
 # Save the config file
-open(CONF, ">$config{'client_conf'}");
+&open_tempfile(CONF, ">$config{'client_conf'}");
 if ($_[0]->{'domain'}) {
 	if ($_[0]->{'broadcast'}) {
-		print CONF "domain $_[0]->{'domain'} broadcast\n";
+		&print_tempfile(CONF, "domain $_[0]->{'domain'} broadcast\n");
 		}
 	else {
 		local @s = @{$_[0]->{'servers'}};
-		print CONF "domain $_[0]->{'domain'} server ",shift(@s),"\n";
+		&print_tempfile(CONF, "domain $_[0]->{'domain'} server ",shift(@s),"\n");
 		foreach $s (@s) {
-			print CONF "ypserver $s\n";
+			&print_tempfile(CONF, "ypserver $s\n");
 			}
 		}
 	}
-close(CONF);
+&close_tempfile(CONF);
 
 # Stop the init script
 local $init = &init_script("ypbind");
@@ -232,11 +232,11 @@ foreach $t (@nis_files) {
 	}
 &flush_file_lines();
 
-open(SLAVES, ">/var/yp/ypservers");
+&open_tempfile(SLAVES, ">/var/yp/ypservers");
 foreach $s (split(/\s+/, $in{'slaves'})) {
-	print SLAVES "$s\n";
+	&print_tempfile(SLAVES, "$s\n");
 	}
-close(SLAVES);
+&close_tempfile(SLAVES);
 
 local $init1 = &init_script("ypserv");
 local $init2 = &init_script("yppasswdd");
@@ -490,9 +490,9 @@ if ($old) {
 	&replace_file_line($rc_config, $old->{'line'}, $line);
 	}
 else {
-	open(RC, ">>$rc_config");
-	print RC $line;
-	close(RC);
+	&open_tempfile(RC, ">>$rc_config");
+	&print_tempfile(RC, $line);
+	&close_tempfile(RC);
 	}
 }
 
