@@ -30,7 +30,7 @@ $POSTFIX_MODULE_VERSION = 5;
 
 do '../web-lib.pl';
 &init_config();
-require '../ui-lib.pl';
+do '../ui-lib.pl';
 %access = &get_module_acl();
 do 'aliases-lib.pl';
 
@@ -198,9 +198,21 @@ sub reload_postfix
     }
 }
 
+# stop_postfix()
+# Attempts to stop postfix, returning undef on success or an error message
+sub stop_postfix
+{
+local $out = `$config{'postfix_control_command'} -c $config_dir stop 2>&1`;
+return $? ? "<tt>$out</tt>" : undef;
+}
 
-
-
+# start_postfix()
+# Attempts to start postfix, returning undef on success or an error message
+sub start_postfix
+{
+local $out = `$config{'postfix_control_command'} -c $config_dir start 2>&1`;
+return $? ? "<tt>$out</tt>" : undef;
+}
 
 # option_radios_freefield(name_of_option, length_of_free_field, [name_of_radiobutton, text_of_radiobutton]+)
 # builds an option with variable number of radiobuttons and a free field
@@ -219,8 +231,8 @@ sub option_radios_freefield
 
     # first radio button (must be default value!!)
     
-    printf "<input type=radio name=$name"."_def value=\"__DEFAULT_VALUE_IE_NOT_IN_CONFIG_FILE__\" %s> $_[2]\n",
-    (&if_default_value($name)) ? "checked" : "";
+    print &ui_oneradio($name."_def", "__DEFAULT_VALUE_IE_NOT_IN_CONFIG_FILE__",
+		       $_[2], &if_default_value($name));
 
     $check_free_field = 0 if &if_default_value($name);
     shift;
@@ -228,18 +240,17 @@ sub option_radios_freefield
     # other radio buttons
     while (defined($_[2]))
     {
-	printf "<input type=radio name=$name"."_def value=\"$_[2]\" %s> $_[3]\n",
-	($v eq $_[2]) ? "checked" : "";
+	print &ui_oneradio($name."_def", $_[2], $_[3], $v eq $_[2]);
 	if ($v eq $_[2]) { $check_free_field = 0; }
 	shift;
 	shift;
     }
 
     # the free field
-    printf "<input type=radio name=$name"."_def value=__USE_FREE_FIELD__ %s>\n",
-	    ($check_free_field == 1) ? "checked" : "";
-    printf "<input name=$name size=$length value=\"%s\"> </td>\n",
-	    ($check_free_field == 1) ? &html_escape($v) : "";
+    print &ui_oneradio($name."_def", "__USE_FREE_FIELD__", undef,
+		       $check_free_field == 1);
+    print &ui_textbox($name, $check_free_field == 1 ? $v : undef, $length);
+    print "</td>\n";
 }
 
 
@@ -255,8 +266,7 @@ sub option_freefield
     printf "<td>".&hlink("<b>$text{$key}</b>", "opt_".$name)."</td> <td %s nowrap>\n",
     $length > 20 ? "colspan=3" : "";
     
-    printf "<input name=$name"."_def size=$length value=\"%s\"> </td>\n",
-	&html_escape($v);
+    print &ui_textbox($name."_def", $v, $length),"</td>\n";
 }
 
 
@@ -273,12 +283,25 @@ sub option_yesno
     :
 	print "<td><b>$text{$key}</b></td> <td nowrap>\n";
     
-    printf "<input type=radio name=$name"."_def value=\"yes\" %s> $text{'yes'}\n",
-    (lc($v) eq "yes") ? "checked" : "";
+    print &ui_radio($name."_def", lc($v),
+		    [ [ "yes", $text{'yes'} ], [ "no", $text{'no'} ] ]);
+    print "</td>\n";
+}
 
-    printf "<input type=radio name=$name"."_def value=\"no\" %s> $text{'no'}\n",
-    (lc($v) eq "no") ? "checked" : "";
+# option_select(name_of_option, &options, [help])
+# Shows a drop-down menu of options
+sub option_select
+{
+    my $name = $_[0];
+    my $v = &get_current_value($name);
+    my $key = 'opts_'.$name;
 
+    defined($_[2]) ?
+	print "<td>".&hlink("<b>$text{$key}</b>", "opt_".$name)."</td> <td nowrap>\n"
+    :
+	print "<td><b>$text{$key}</b></td> <td nowrap>\n";
+    
+    print &ui_select($name."_def", lc($v), $_[1]);
     print "</td>\n";
 }
 
@@ -435,6 +458,13 @@ sub regenerate_virtual_table
     &regenerate_any_table($virtual_maps);
 }
 
+# regenerate_header_table
+#
+sub regenerate_header_table
+{
+    &regenerate_any_table("header_checks");
+}
+
 
 # regenerate_canonical_table
 #
@@ -696,6 +726,31 @@ if (!defined(@mail_system_cache)) {
 return wantarray ? @mail_system_cache : $mail_system_cache[0];
 }
 
+# list_queue()
+# Returns a list of strutures, each containing details of one queued message
+sub list_queue
+{
+local @qfiles;
+open(MAILQ, "$config{'mailq_cmd'} 2>/dev/null |");
+while(<MAILQ>) {
+	next if (/^(\S+)\s+is\s+empty/i || /^\s+Total\s+requests:/i);
+	if (/^([^\s\*]+)\*?\s*(\d+)\s+(\S+\s+\S+\s+\d+\s+\d+:\d+:\d+)\s+(.*)/) {
+		push(@qfiles, { 'id' => $1,
+			        'size' => $2,
+				'date' => $3,
+				'from' => $4 });
+		}
+	elsif (/\((.*)\)/ && @qfiles) {
+		$qfiles[$#qfiles]->{'status'} = $1;
+		}
+	elsif (/^\s+(\S+)/ && @qfiles) {
+		$qfiles[$#qfiles]->{'to'} .= "$1 ";
+		}
+	}
+close(MAILQ);
+return @qfiles;
+}
+
 # parse_queue_file(id)
 # Parses a postfix mail queue file into a standard mail structure
 sub parse_queue_file
@@ -772,7 +827,7 @@ else {
 sub before_save
 {
 if ($config{'check_config'} && !defined($save_file)) {
-	$save_file = &tempname();
+	$save_file = &transname();
 	system("cp $config{'postfix_config_file'} $save_file");
 	}
 }
@@ -807,7 +862,7 @@ sub ensure_map
 {
 foreach my $mf (&get_maps_files(&get_real_value($_[0]))) {
 	if ($mf =~ /^\// && !-e $mf) {
-		open(TOUCH, ">$mf");
+		open(TOUCH, ">$mf") || &error(&text("efilewrite", $mf, $!));
 		close(TOUCH);
 		chmod(0755, $mf);
 		}
