@@ -687,9 +687,9 @@ if ($config{'test_always'}) {
 		# Something failed .. revert all files
 		local $f;
 		foreach $f (keys %before_changing) {
-			open(AFTER, ">$f");
-			print AFTER $before_changing{$f};
-			close(AFTER);
+			&open_tempfile(AFTER, ">$f");
+			&print_tempfie(AFTER, $before_changing{$f});
+			&close_tempfile(AFTER);
 			}
 		&error(&text('eafter', "<pre>$err</pre>"));
 		}
@@ -763,5 +763,16 @@ else {
 return ($inet, $inet_mod);
 }
 
+# get_proftpd_pid()
+sub get_proftpd_pid
+{
+if ($config{'pid_file'}) {
+	return &check_pid_file($config{'pid_file'});
+	}
+else {
+	return &find_byname("proftpd");
+	}
+}
+
 1;
 
