@@ -537,6 +537,10 @@ return ( $_[0] =~ /^(\S+):/ ? "$1" : $_[0],
 @rules_actions = ( 'ACCEPT', 'DROP', 'REJECT', 'DNAT', 'DNAT-', 'REDIRECT' );
 if (&version_atleast(2, 0, 0)) {
 	push(@rules_actions, 'CONTINUE');
+	push(@rules_actions, 'ACCEPT+');
+	push(@rules_actions, 'NONAT');
+	push(@rules_actions, 'REDIRECT-');
+	push(@rules_actions, 'LOG');
 	}
 @rules_protos = ( 'all', 'related', 'tcp', 'udp', 'icmp' );
 
@@ -549,7 +553,9 @@ local ($action, $log) = split(/:/, $_[0]);
 print "<tr> <td><b>$text{'rules_0'}</b></td>\n";
 print "<td colspan=3><select name=action>\n";
 $found = !$_[0];
-foreach $a (@rules_actions) {
+foreach $a ((sort { $a cmp $b } @rules_actions),
+	    "--------",
+	    &list_standard_actions()) {
 	printf "<option value=%s %s>%s\n",
 		$a, $action eq $a ? "selected" : "", $a;
 	$found++ if ($action eq $a);
@@ -661,6 +667,7 @@ if (&version_atleast(1, 4, 7)) {
 
 sub rules_validate
 {
+$in{'action'} !~ /----/ || &error($text{'rules_eaction'});
 $in{'source'} || $in{'sother'} =~ /^\S+$/ || &error($text{'rules_esother'});
 !$in{'sinzone_def'} || $in{'sinzone'} =~ /\S/ || &error($text{'rules_esinzone'});
 $in{'dest'} || $in{'dother'} =~ /^\S+$/ || &error($text{'rules_edother'});
@@ -1241,5 +1248,44 @@ else {
 	}
 }
 
+# run_before_apply_command()
+# Runs the before-applying command, if any. If it failes, returns the error
+# message output
+sub run_before_apply_command
+{
+if ($config{'before_apply_cmd'}) {
+	local $out = &backquote_logged("($config{'before_apply_cmd'}) </dev/null 2>&1");
+	return $out if ($?);
+	}
+return undef;
+}
+
+# run_after_apply_command()
+# Runs the after-applying command, if any
+sub run_after_apply_command
+{
+if ($config{'after_apply_cmd'}) {
+	&system_logged("($config{'after_apply_cmd'}) </dev/null >/dev/null 2>&1");
+	}
+}
+
+# list_standard_actions()
+# Returns a list of standard Shorewall actions
+sub list_standard_actions
+{
+local @rv;
+open(ACTIONS, $config{'actions'});
+while(<ACTIONS>) {
+	s/\r|\n//g;
+	s/#.*$//;
+	s/\s+$//;
+	if (/\S/) {
+		push(@rv, $_);
+		}
+	}
+close(ACTIONS);
+return @rv;
+}
+
 1;
 
