@@ -54,6 +54,6 @@ $_[0]->{'bootup'} = $in{'bootup'};
 
 sub get_icons
 {
-return ( "access" ,"bind" ,"ui" ,"mods" ,"os" ,"lang" ,"upgrade" ,"session" ,"assignment" ,"categories" ,"themes" ,"ssl" ,"configs" ,"acl" ,"restrict" ,"users" ,"defacl", "sessions" );
+return ( "access" ,"bind" ,"ui" ,"umods" ,"os" ,"lang" ,"upgrade" ,"session" ,"assignment" ,"categories" ,"themes", "referers", "anon", "ssl" ,"configs" ,"acl" ,"restrict" ,"users" ,"defacl", "sessions" );
 }
 
