@@ -3,7 +3,7 @@
 # XXX share text with webmin!
 
 do '../web-lib.pl';
-&init_config();
+{'usermin_dir'}&init_config();
 require '../ui-lib.pl';
 %access = &get_module_acl();
 &foreign_require("webmin", "webmin-lib.pl");
@@ -22,6 +22,8 @@ $latest_tgz = "http://www.webmin.com/download/usermin-latest.tar.gz";
 
 $default_key_size = 512;
 
+$cron_cmd = "$module_config_directory/update.pl";
+
 # get_usermin_miniserv_config(&array)
 sub get_usermin_miniserv_config
 {
@@ -223,9 +225,9 @@ return "$config{'usermin_dir'}/webmin.acl";
 # save_usermin_acl(user, &modules)
 sub save_usermin_acl
 {
-open(ACL, ">".&usermin_acl_filename());
-print ACL $_[0],": ",join(" ", @{$_[1]}),"\n";
-close(ACL);
+&open_tempfile(ACL, ">".&usermin_acl_filename());
+&print_tempfile(ACL, $_[0],": ",join(" ", @{$_[1]}),"\n");
+&close_tempfile(ACL);
 }
 
 # install_usermin_module(file, unlink, nodeps)
@@ -246,8 +248,8 @@ if ($two eq "\037\235") {
 		unlink($file) if ($need_unlink);
 		return &text('install_ecomp', "<tt>uncompress</tt>");
 		}
-	local $temp = $file =~ /\/([^\/]+)\.Z/i ? &tempname("$1")
-						: &tempname();
+	local $temp = $file =~ /\/([^\/]+)\.Z/i ? &transname("$1")
+						: &transname();
 	local $out = `uncompress -c "$file" 2>&1 >$temp`;
 	unlink($file) if ($need_unlink);
 	if ($?) {
@@ -262,8 +264,8 @@ elsif ($two eq "\037\213") {
 		unlink($file) if ($need_unlink);
 		return &text('install_egzip', "<tt>gunzip</tt>");
 		}
-	local $temp = $file =~ /\/([^\/]+)\.gz/i ? &tempname("$1")
-						 : &tempname();
+	local $temp = $file =~ /\/([^\/]+)\.gz/i ? &transname("$1")
+						 : &transname();
 	local $out = `gunzip -c "$file" 2>&1 >$temp`;
 	unlink($file) if ($need_unlink);
 	if ($?) {
@@ -307,16 +309,16 @@ if ($type eq 'rpm' && $file =~ /\.rpm$/i &&
 		# Update the ACL for the usermin user
 		local %acl;
 		&read_usermin_acl(undef, \%acl);
-		open(ACL, "> ".&usermin_acl_filename());
+		&open_tempfile(ACL, "> ".&usermin_acl_filename());
 		foreach $u (keys %acl) {
 			local @mods = @{$acl{$u}};
 			if ($u eq 'user') {
 				push(@mods, $name);
 				@mods = &unique(@mods);
 				}
-			print ACL "$u: ",join(' ', @mods),"\n";
+			&print_tempfile(ACL, "$u: ",join(' ', @mods),"\n");
 			}
-		close(ACL);
+		&close_tempfile(ACL);
 		&webmin_log("install", undef, $name,
 			    { 'desc' => $mdescs[0] });
 		}
@@ -361,7 +363,7 @@ else {
 
 	# Get the module.info files to check dependancies
 	local $ver = &get_usermin_version();
-	local $tmpdir = &tempname();
+	local $tmpdir = &transname();
 	mkdir($tmpdir, 0700);
 	local $err;
 	local @realmods;
@@ -461,16 +463,16 @@ else {
 	# Update ACL for this user so they can access the new modules
 	local %acl;
 	&read_usermin_acl(undef, \%acl);
-	open(ACL, "> ".&usermin_acl_filename());
+	&open_tempfile(ACL, "> ".&usermin_acl_filename());
 	foreach $u (keys %acl) {
 		local @mods = @{$acl{$u}};
 		if ($u eq 'user') {
 			push(@mods, @realmods);
 			@mods = &unique(@mods);
 			}
-		print ACL "$u: ",join(' ', @mods),"\n";
+		&print_tempfile(ACL, "$u: ",join(' ', @mods),"\n");
 		}
-	close(ACL);
+	&close_tempfile(ACL);
 	}
 unlink("$config{'usermin_dir'}/module.infos.cache");
 
@@ -496,11 +498,12 @@ return @rv;
 # Saves the list of additional module restrictions
 sub save_usermin_usermods
 {
-open(USERMODS, ">$config{'usermin_dir'}/usermin.mods");
+&open_tempfile(USERMODS, ">$config{'usermin_dir'}/usermin.mods");
 foreach $u (@{$_[0]}) {
-	printf USERMODS "%s:%s:%s\n", $u->[0], $u->[1], join(" ", @{$u->[2]});
+	&print_tempfile(USERMODS,
+		join(":", $u->[0], $u->[1], join(" ", @{$u->[2]})),"\n");
 	}
-close(USERMODS);
+&close_tempfile(USERMODS);
 }
 
 # get_usermin_miniserv_users()
@@ -528,13 +531,14 @@ sub save_usermin_miniserv_users
 {
 local %miniserv;
 &get_usermin_miniserv_config(\%miniserv);
-open(USERS, ">$miniserv{'userfile'}");
+&open_tempfile(USERS, ">$miniserv{'userfile'}");
 local $u;
 foreach $u (@_) {
-	print USERS join(":", $u->{'name'}, $u->{'pass'}, $u->{'sync'},
-			      $u->{'cert'}, $u->{'allow'}),"\n";
+	&print_tempfile(USERS,
+		join(":", $u->{'name'}, $u->{'pass'}, $u->{'sync'},
+			  $u->{'cert'}, $u->{'allow'}),"\n");
 	}
-close(USERS);
+&close_tempfile(USERS);
 }
 
 # can_use_module(module)
@@ -559,5 +563,80 @@ sub base_version
 return sprintf("%.2f0", $_[0]);
 }
 
+# find_cron_job(\@jobs)
+sub find_cron_job
+{
+local ($job) = grep { $_->{'user'} eq 'root' &&
+		      $_->{'command'} eq $cron_cmd } @{$_[0]};
+return $job;
+}
+
+# delete_usermin_module(module, [delete-acls])
+# Deletes some usermin module, clone or theme, and return a description of
+# the thing deleted.
+sub delete_usermin_module
+{
+local $m = $_[0];
+return undef if (!$m);
+local %minfo = &get_usermin_module_info($m);
+%minfo = &get_usermin_theme_info($m) if (!%minfo);
+return undef if (!%minfo);
+local ($mdesc, @aclrm);
+@aclrm = ( $m ) if ($_[1]);
+local %miniserv;
+&get_usermin_miniserv_config(\%miniserv);
+local %ugconfig;
+&get_usermin_config(\%uconfig);
+local $mdir = "$miniserv{'root'}/$m";
+local $cdir = "$config{'usermin_dir'}/$m";
+if ($minfo{'clone'}) {
+	# Deleting a clone
+	local %cinfo;
+	&read_file("$config{'usermin_dir'}/$m/clone", \%cinfo);
+	unlink($mdir);
+	system("rm -rf ".quotemeta($cdir));
+	if ($ugconfig{'theme'}) {
+		unlink("$miniserv{'root'}/$ugconfig{'theme'}/$m");
+		}
+	$mdesc = &webmin::text('delete_desc1', $minfo{'desc'}, $minfo{'clone'});
+	}
+else {
+	# Delete any clones of this module
+	local @clones;
+	local @mst = stat($mdir);
+	opendir(DIR, $miniserv{'root'});
+	local $l;
+	foreach $l (readdir(DIR)) {
+		@lst = stat("$miniserv{'root'}/$l");
+		if (-l "$miniserv{'root'}/$l" && $lst[1] == $mst[1]) {
+			unlink("$miniserv{'root'}/$l");
+			system("rm -rf $config{'usermin_dir'}/$l");
+			push(@clones, $l);
+			}
+		}
+	closedir(DIR);
+
+	open(TYPE, "$mdir/install-type");
+	chop($type = <TYPE>);
+	close(TYPE);
+
+	# Deleting the real module
+	local $size = &disk_usage_kb($mdir);
+	$mdesc = &webmin::text('delete_desc2', "<b>$minfo{'desc'}</b>",
+			   "<tt>$mdir</tt>", $size);
+	if ($type eq 'rpm') {
+		# This module was installed from an RPM .. rpm -e it
+		&system_logged("rpm -e ubm-$m");
+		}
+	else {
+		# Module was installed from a .wbm file .. just rm it
+		&system_logged("rm -rf ".quotemeta($mdir));
+		}
+	}
+
+&webmin_log("delete", undef, $m, { 'desc' => $minfo{'desc'} });
+return $mdesc;
+}
+
 1;
 
